package cn.ecpark.service.usergw;

import java.util.Map;

import com.alibaba.fastjson.JSON;

import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.context.SpringBootTest.WebEnvironment;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.test.context.TestPropertySource;
import org.springframework.test.context.junit4.SpringRunner;

import reactor.netty.http.client.HttpClient;
import reactor.netty.http.client.HttpClient.ResponseReceiver;
import reactor.netty.http.client.HttpClientResponse;

// TODO: 添加附加Dubbo Service for Test启动进程
@SpringBootTest(classes = App.class, webEnvironment = WebEnvironment.DEFINED_PORT)
@RunWith(SpringRunner.class)
@TestPropertySource(locations = { "application3.properties" })
public class TestHttp2DubboConfig3 {

    @LocalServerPort
    private int serverPort;

    @Test
    public void Test2RequestHttp2DubboSetHeader() {

        // Test Base Url
        HttpClient client = HttpClient.create();
        ResponseReceiver<?> receiver;
        String content;

        receiver = client.baseUrl("http://localhost:" + serverPort + "/dubbo/hello")
                .headers(h -> h.set("method", "Hello")).get();

        HttpClientResponse response = receiver.response().block();
        Assert.assertNotNull(response); 
        Assert.assertEquals(response.status().code(), 200);
        content = receiver.responseContent().asString().blockLast();
        Assert.assertNotNull(content); 
        Assert.assertEquals(content, "Hello Dubbo");
    }


    @Test
    @SuppressWarnings("unchecked")
    public void Test3RestfulFilters() {

        // Test Base Url
        HttpClient client = HttpClient.create();
        ResponseReceiver<?> receiver;
        String content;
        
        // Test 302
        receiver = client.baseUrl("http://localhost:" + serverPort + "/http/xixi").get();
        HttpClientResponse response = receiver.response().block();
        Assert.assertNotNull(response); 
        Assert.assertEquals(response.status().code(), 302);
        content = receiver.responseContent().asString().blockLast();
        Assert.assertNull(content); 

        // Test RedirectTo
        receiver = client.baseUrl("http://localhost:" + serverPort + "/http/xixi").followRedirect(true).get();
        response = receiver.response().block();    
        Assert.assertEquals(response.status().code(), 200);
        content = receiver.responseContent().asString().blockLast();
        Assert.assertEquals(content, "Http Hello");

        // 测试没配置的路由
        receiver = client.baseUrl("http://localhost:" + serverPort + "/xixi").followRedirect(true).get();
        response = receiver.response().block();    
        Assert.assertEquals(response.status().code(), 404);
        content = receiver.responseContent().asString().blockLast();
        Assert.assertNotNull(content);

        Object rjson = JSON.parse(content);
        Map<String, Object> rmap = (Map<String, Object>)rjson;
        Assert.assertEquals(rmap.get("error").toString(), "Not Found");

        // 测试普通路由
        receiver = client.baseUrl("http://localhost:" + serverPort + "/http/hello").get();
        response = receiver.response().block(); 
        Assert.assertEquals(response.status().code(), 200);
        content = receiver.responseContent().asString().blockLast();
        Assert.assertNotNull(content);       

        // 测试普通路由
        receiver = client.baseUrl("http://localhost:" + serverPort + "/http/rewrite").get();
        response = receiver.response().block(); 
        Assert.assertEquals(response.status().code(), 200);
        content = receiver.responseContent().asString().blockLast();
        Assert.assertEquals(content, "Http Hello");     


    }

 
}